;this curve plots the photocurve for a burst mode interval
;and fits two exponentials to the curve giving you the photoelectron currents
;and the photoelectron temperatures
;this complements the photocurve.pro which 
;calculates the photocurve for large intervals this is more suited to shorter intervals
pro Photo_Curve_Local


mms_init
  ;set local data directory
  local_data_dir='C:\Users\oroberts\IDLWorkspace\'

;time range
trange=['2016-10-16/20:32:34.00','2016-10-16/20:37:40.0']

;select spacecraft
probes='2'
;load fpi data
  mms_load_fpi, trange=trange, probes=probes, level='l2', data_rate='brst', datatype='des-moms', local_data_dir = local_data_dir,/time_clip

  ;load edp data
  mms_load_edp, trange=trange, probes=probes, level='l2', data_rate='brst', datatype='scpot', local_data_dir = local_data_dir,/time_clip
 


  ;get density,scpot and temperature data
get_data, 'mms2_des_numberdensity_brst', tne, n_e
get_data,'mms2_des_numberdensity_err_brst',terr,n_e_error

get_data,'mms2_des_temptensor_err_brst',terrTen,TTenErr
get_data, 'mms2_fgm_b_gse_brst_l2_bvec', tmag,bfield

get_data, 'mms2_des_temppara_brst', tTe, Tepar

get_data, 'mms2_des_tempperp_brst', tperp, Teperp
get_data,'mms2_edp_scpot_brst_l2',tscp,scpot


TparError=dblarr(n_elements(terrTen))
TperpError=dblarr(n_elements(terrTen))



for i = 0, n_elements(terrTen)-1 do begin
 
  
TBunit=reform(reform(TTenErr(i,*,*)) ## transpose(Bunit))
TparError(i)=(Bunit ## transpose(TBunit))
TperpError(i)=(trace(reform(TTenErr(i,*,*)))-TparError(i))/2.0

endfor

Te=(Teperp*2.d +Tepar)/3.d
TeError=sqrt((4.*TperpError^2.)/(9.)+(1.*TparError^2.)/(9.))







  
  window, 1

  ;fundamental constants
  ;fundamental Charge      ;electron mass           ;Boltzmann Constant
  q=double(1.6021765e-19)&me=double(9.10938356e-31)&kB=double(1.38064852e-23)

  ;spacecraft properties
  Aspac=34.D0 ; spacecraft Area in m^2
  Csc=double(2.e-9) ;spacecraft Capacitance in F


  ;calculate mean electron temperature (from FPI)
   ;convert to Kelvin
  meanTe=mean(Te)
  Te_K=meanTE*11604.505d

  ;photoelectron parameter Guesses
  Iph0=double(570.) ; micro A
  Iph1=double(50.) ;micro  A
  Tph0=1.8d ;eV
  Tph1=6.6d ;eV
  
  ;set initial parameters for iteration of photocurve
  initial_parameters=[Iph0,q/(kB*Tph0*11604.505d),Iph1,q/(kB*Tph1*11604.505d)]

  parinfo=REPLICATE({value:0.D,limited:[0,0],limits:[0.D,0.D],step:[0,0],fixed:0},4)
  ;change here to fix any parameters
  fix_parameters=[0,0,0,0]
  parinfo[0].fixed = UINT(fix_parameters[0])
  parinfo[0].limited = [1,0]
  parinfo[0].limits = [0.0001,50000]
  parinfo[1].fixed = UINT(fix_parameters[1])
  parinfo[2].fixed = UINT(fix_parameters[2])
  parinfo[3].fixed = UINT(fix_parameters[3])
  parinfo[*].value = initial_parameters

;resample the spacecraft potential to the times of the direct density measurement
  scpotLowRes=resample(tscp-min(tne),scpot,tne-min(tne))


;determine the electron Thermal Current
  Ie=10d^12*(Aspac*q*n_e*sqrt(kb*Te*11604.505d))/(sqrt(2.d*me*!PI))*(1.d + (q*scpotLowRes)/(kb*Te*11604.505d))
  
  ;error estimation on Ie
  partialIePartialne=10d^12*(Aspac*q*sqrt(kb*Te*11604.505d))/(sqrt(2.d*me*!PI))*(1.d + (q*scpotLowRes)/(kb*Te*11604.505d))
partialIePartialTe=10d^12*(Aspac*q*sqrt(kb*11604.505d))/(2.0*sqrt(2.d*me*!PI*Te))*(1.d + 0.5*Te^(-3./2.)*(q*scpotLowRes)/(kb*11604.505d))

ErrIe=sqrt(partialIePartialne^2.*n_e_error^2.+partialIePartialTe^2.*TeError)




 greekmu='!4' + String("154B) + '!X'


;warning trigger if range of SC potentials is too small.
if (max(scpotLowRes)-min(scpotLowRes)) LT 8. then begin
  print, '!!WARNING!! range of spacecraft potentials is too low. Use a longer time range to ensure good determination of the photocurve.'

  endif 


;bin spacecraft potential 
ScpotBins=(((max(scpotLowRes)-min(scpotLowRes))/61)*findgen(61))+min(scpotlowres)
ScpotBinCentres=dblarr(60)


  binnedIe=dblarr(60)
  errorIe=dblarr(60)

  for kk=0,59 do begin
    ScpotBinCentres(kk)=(SCpotBins(kk)+SCpotBins(kk+1))/2.
    index=where(scpotLowRes ge ScpotBins(kk) and scpotLowRes lt ScpotBins(kk+1))
    if index(0) ne -1 then begin
      binnedIe(kk)=median(Ie(index))
      errorIe(kk)=1.4826*median(abs(Ie(index)-binnedIe(kk)))

    endif else begin
      binnedIe(kk)=-1.0
      errorIe(kk)=-1.0
    endelse
  endfor
stop
  index=where(binnedIe gt 0.0 and errorIe gt 0.0)
  errorIe=errorIe(index)
  binnedIe=binnedie(index)
  ScpotBinCentres=ScpotBinCentres(index)

;plot, the photocurve
!p.multi=0
  plot, scpotLowRes,Ie,psym=3,title=trange[0]+'-'+trange[1],xtitle='Spacecraft Potential (V)', ytitle='I!De!N ['+greekmu+' A]',yrange=[0,40]
  
  ;fit the photoelectron parameters
  Pars=mpfitfun('doubleexp',scpotLowRes(sort(scpotLowRes)),Ie(sort(scpotLowRes)),ErrIe(sort(scpotLowRes)),PARINFO=parinfo,weights=(1./2*ErrIe(sort(scpotLowRes))^2.))
  ParsBinned=mpfitfun('doubleexp',scpotbincentres,binnedie,errorIe,PARINFO=parinfo,weights=(1./2*errorIe^2.))
  
  print, ParsBinned(0)*exp(-ParsBinned(1)*8)
  stop
  ;plot the fitting.
 oplot, findgen(1000)/50, doubleexp(findgen(1000)/50,Pars),color=fsc_color('cyan')
 oplot, findgen(1000)/50, doubleexp(findgen(1000)/50,Parsbinned),color=fsc_color('maroon')
 oplot, ScpotBinCentres,binnedIe,color=fsc_color('red'),psym=7
 oploterror, ScpotBinCentres,binnedIe,errorIe,color=fsc_color('red')

;print out the photoelectron parameters for the data
 print, '   Iph0=',string(Pars(0))+' micro A',' Tph0='+string(q/(kB*Pars(1))/11604.505d)+'eV   Iph1='+string(Pars(2)*1e-6)+'micro A'+'  Tph1='+string(q/(kB*Pars(3))/11604.505d)+'eV'
 ;and the binned data
  print, '   Iph0=',string(ParsBinned(0))+' micro A',' Tph0='+string(q/(kB*ParsBinned(1))/11604.505d)+'eV   Iph1='+string(ParsBinned(2)*1e-6)+'micro A'+'  Tph1='+string(q/(kB*ParsBinned(3))/11604.505d)+'eV'

  
  Iph0=Pars(0)*1e-6
  Tph0=q/(kB*Pars(1))/11604.505d
  Iph1=Pars(2)*1e-6
  Tph1=q/(kB*Pars(3))/11604.505d


  
  ;find the density from sc potential
  term1=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
  term2=(1.0d/((1.0d +(q*scpot)/(kB*meanTE*11604.505d))))
  term3=Iph0*exp(-q*double(scpot)/(kB*TPh0*11604.505d))
  term4=Iph1*exp(-q*double(scpot)/(kB*TPh1*11604.505d))

;combine all the terms to give you electron density in cm^-3
  nsc=(term1*term2*(term3+term4))*1e-6
  
 
;plot to compare FPI and estimation from SC potential

  plot, tne-min(tne), n_e, xtitle='Time [s]', ytitle='n!De!N [cm^-3]' , title=trange[0]+'-'+trange[1],/ylog,psym=3
  oplot, tscp-min(tne),nsc,color=fsc_color('red'),psym=3

  end