;+
; :Description:
;    Describe the procedure.
;
;this code performs a correction to the 
;spacecraft potential removing the contributions of electric fields
;from the potential so that it can be used for density estimation
;
;
;
; :Author: oroberts
;-
pro SC_Potential_correction_Fin

 mms_init
local_data_dir='C:\Users\oroberts\IDLWorkspace\'

trange=['2016-10-16/20:33:40.50','2016-10-16/20:33:41.5']
probes='2'
;example load data from fpi
mms_load_fpi, trange=trange, probes=probes, level='l2', data_rate='brst', datatype='des-moms', local_data_dir = local_data_dir,/time_clip
mms_load_fpi, trange=trange, probes=probes, level='l2',data_rate='brst', datatype='dis-moms',local_data_dir = local_data_dir,/time_clip

;load edp data
mms_load_edp, trange=trange, probes=probes, level='l2', data_rate='brst', local_data_dir = local_data_dir,/time_clip
mms_load_edp, trange=trange, probes=probes, level='l2', data_rate='brst', datatype='scpot', local_data_dir = local_data_dir,/time_clip
mms_load_edp, trange=trange, probes=probes, level='l2pre', data_rate='brst',datatype='dce2d', local_data_dir = local_data_dir,/time_clip
mms_load_edp, trange=trange, probes=probes, level='l2', data_rate='brst', datatype='hmfe', local_data_dir = local_data_dir,/time_clip

  mms_load_fpi, trange=trange, probes=probes, data_rate='brst',datatype='des-partmoms', local_data_dir = local_data_dir,/time_clip

 mms_load_fgm,  trange=trange,probes=probes, level='l2',data_rate='brst', local_data_dir = local_data_dir,/time_clip



;get density, velocity data data
get_data, 'mms2_des_numberdensity_brst', tne, n_e
get_data,'mms2_des_numberdensity_err_brst',terr,n_e_error

get_data,'mms2_des_numberdensity_part_brst',te2,n_epart2, dlimits=des2_metadata

get_data, 'mms2_dis_numberdensity_brst', tni, n_i
get_data,'mms2_dis_numberdensity_err_brst',tirr,n_i_error
get_data, 'mms2_des_bulkv_gse_brst',tvel,vele

;get temperature data
get_data, 'mms2_des_temppara_brst', tTe, Tepar
get_data, 'mms2_des_tempperp_brst', tperp, Teperp
;get efield data
get_data,'mms2_edp_hmfe_dsl_brst_l2',tedp,efield
;;stop
;get FGM data 
get_data, 'mms2_fgm_b_gse_brst_l2', tmag,bfield
;get spacecraft potential
get_data,'mms2_edp_scpot_brst_l2',tscp,scpot
get_data,'mms'+probes+'_edp_phase_brst_l2pre',tphb,phaseb



timePotSecs=tscp-min(tscp)
TimeElecSecs=tedp-min(tedp)
TimeElecSecs1=tscp-min(tedp)


B0=[mean(bfield(*,0)),mean(bfield(*,1)),mean(bfield(*,2))]


Print,'B0r=',sqrt(B0(0)^2.+B0(1)^2.+B0(2)^2.)
Ve=[mean(vele(*,0)),mean(vele(*,1)),mean(vele(*,2))]
Bunit=B0/(sqrt(B0(0)^2.+B0(1)^2.+B0(2)^2.))
Vunit=Ve/(sqrt(Ve(0)^2.+Ve(1)^2.+Ve(2)^2.))

xunit=[0.0,1.0,0.0]
xunit=[1.0,0.0,0.0]
ez=Bunit
ex=crossp(ez,xunit)
ey=crossp(ez,ex)

;mean field aligned coordinates





Te=(Teperp*2.d +Tepar)/3.d

;fundamental constants
q=double(1.6021765e-19)&me=double(9.10938356e-31)&kB=double(1.38064852e-23)

;spacecraft properties
Aspac=34.0 ; spacecraft Area in m^2
Csc=double(2.e-9) ;spacecraft Capacitance in F


  ;photoelectron parameters
  ; get these from code photocurve local
Iph0=    0.0013745335;current in micro amps
Tph0=       1.5414510;photoelectron temperature population 1
Iph1=  7.3789281e-005;current in micropamps
Tph1=       7.3684482;photoelectron temperature population 2



;choose to low pass filter at 1kHz or not
filter1=0
if filter1 eq 1 then begin
  Coeff = DIGITAL_FILTER(0.0, 1./10., 50, 20)
  ; Apply the filter:
  Yout = CONVOL(scpot, Coeff)
  scpot=Yout
endif

;calculate mean electron temperature (from FPI)
meanTe=mean(Te)

;convert to Kelvin
Te_K=meanTE*11604.505d

;Time constant
deltaT_RC=Csc*1.D0/(abs(((-1.0*Iph0*q)/(kB*Tph0*11604.505d))*exp(-q*double(scpot)/(kB*Tph0*11604.505d))-(((1.0*Iph1*q)/(kB*Tph1*11604.505d))*exp(-q*double(scpot)/(kB*Tph1*11604.505d)))))
dt=tscp(1)-tscp(0)

;smoothing window size based on 10 times spacecraft max time constant

smoothingwindow=fix(10.0*max(deltaT_RC)/dt)

index=where(scpot eq 0)
scpot(index)=mean(scpot)



;canculate unchange electron density from sc potential
term1=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
term2=(1.0d/((1.0d +(q*scpot)/(kB*meanTE*11604.505d))))
term3=Iph0*exp(-q*double(scpot)/(kB*TPh0*11604.505d))
term4=Iph1*exp(-q*double(scpot)/(kB*TPh1*11604.505d))



nsc=term1*term2*(term3+term4)


;magnitude of electric field AC fluctuations
emag1=sqrt(efield(*,0)^2.+efield(*,1)^2.+efield(*,2)^2.)

efieldpar=efield(*,0)*ez(0)+efield(*,1)*ez(1)+efield(*,2)*ez(2)
efieldperp=efield(*,0)*ex(0)+efield(*,1)*ex(1)+efield(*,2)*ex(2)
efieldperp2=efield(*,0)*ey(0)+efield(*,1)*ey(1)+efield(*,2)*ey(2)
thisLetter = "136B
perpSymbol = '!9' + String(thisLetter) + '!X'


thisLetter2 = "347B
parallelsymbol = '!9' + String(thisLetter2) + '!X'

emag=sqrt(efieldpar^2.+efieldperp^2.+efieldperp2^2.)
!p.charsize=2
!p.multi=[0,1,4]
window, xsize=700,ysize=1200,/free
;plot the electric field waveforms

plot,TimeElecSecs, efieldpar,color=fsc_color('black'),yrange=[-120,120],/nodata,ytitle='Electric Field (mV/m)',title='MMS'+probes+' '+trange(0)+'-'+trange(1)
oplot,TimeElecSecs, efieldpar,color=fsc_color('red')
oplot,TimeElecSecs,efieldperp2,color=fsc_color('blue')
oplot,TimeElecSecs,efieldperp,color=fsc_color('black')
xyouts,-0.12,120,'(a)',/data
xyouts,0.05,70,'E!D'+parallelSymbol+parallelSymbol+'!N',color=fsc_color('red')
xyouts,0.12,70,'E!D'+perpsymbol+' 1!N' ,color=fsc_color('black')
xyouts,0.22,70,'E!D'+perpsymbol+' 2!N' ,color=fsc_color('blue')


;calculate the enevelope of these fluctuations
;using hilbert transform
EnvX=envelope_calc(efield(*,0))
EnvY=envelope_calc(efield(*,1))
Envz=envelope_calc(efield(*,2))

EnvTot=sqrt(EnvX^2.+EnvY^2.+EnvZ^2.)
plot,TimeElecSecs, EnvTot, ytitle='Electric field Envelope (mV/m)'
xyouts,-0.12,140,'(b)',/data

envR=resample(tedp,EnvTot,tscp)
;envRShift=real_part(fft(fft(envR)*exp(complex(0.0,-1.0)*[phase,-1.0*reverse(phase)]),/inverse))
;here we find the time delay between the Eenv and SC potential signals
;define a vector of lags (in data units)
lags=findgen(n_elements(scpot))-float(n_elements(scpot)/2)
;perform the cross correlation
ccor=c_correlate(envR,scpot,lags)
;find value with the best correlation
maxcorr=max(ccor,indexmax)
;find the value of the lag
shiftval=(indexmax-n_elements(scpot)/2)


plot, timePotSecs,scpot,ytitle='Spacecraft Potential (V)',yrange=[min(scpot),max(scpot)]
xyouts,-0.12,max(scpot),'(c)',/data
xyouts, 0.02,15.5,'CC='+string(maxcorr,Format='(D0.6)'),/data
xyouts, 0.02,14.5,'dt='+string(shiftval*dt,Format='(D0.6)')+'s',/data

plot, tscp-min(tscp),nsc*1e-6,xtitle='Time (s)',ytitle='n!De!N (cm!U-3!N)',yrange=[0.00,0.7]
oplot, tni-min(tscp),n_i,color=fsc_color('blue')
errplot, tni-min(tscp),n_i+n_i_error,n_i-n_i_error,color=fsc_color('blue')
oplot, tne-min(tscp),n_e,color=fsc_color('green')
errplot, tne-min(tscp),n_e+n_e_error,n_e-n_e_error,color=fsc_color('green')
xyouts,0.1,0.26,'n!DSC!N',/data

xyouts,0.1,0.19,'n!De!N FPI-DES',color=fsc_color('green'),/data
xyouts,0.1,0.12,'n!Di!N FPI-DIS',color=fsc_color('blue'),/data

xyouts,-0.12,0.7,'(d)',/data



;bin the data
efieldBins=findgen(125)+min(envR)


binnedscpot=dblarr(124)
errorscpot=dblarr(124)
bincentres=findgen(124)+3
for kk=0,123 do begin
  index=where(envR ge efieldbins(kk) and envR lt efieldbins(kk+1))
  if index(0) ne -1 then begin
  binnedscpot(kk)=median(scpot(index))
  errorscpot(kk)=1.4826*median(abs(scpot(index)-binnedscpot(kk)))

  endif else begin
    binnedscpot(kk)=-1.0
    errorscpot(kk)=-1.0
  endelse
endfor

;remove bins with no counts
index=where(binnedscpot gt 0.0 and errorscpot gt 0.0)
errorscpot=errorscpot(index)


binnedscpot=binnedscpot(index)
bincentres=bincentres(index)


;convert SC potential to a corresponding Electric field using linear regression
result=linfit(bincentres, binnedscpot,measure_errors=errorscpot,chisqr=chisqr)
p=[10.,0.01,13.5]
model=result(1)*findgen(150)+result(0)

;convert using polynomial

fitpars=mpfitfun('polycurve1',bincentres, binnedscpot,errorscpot,p)


;use parameters derived from spinrem routine
restore,filename='C:\Users\oroberts\Desktop\SolarWindDensity\fittedspinparametersINT1SC_PotCorr.sav',/verbose

;electric field calibrated to potential units using a polynomial fit no basline 
ElecCal=(fitpars(0)*envR^2.+fitpars(1)*envR)
;electric field calibrated with polynomial fit with baseline
ElecCal2=(fitpars(0)*envR^2.+fitpars(1)*envR+fitpars(2))
corrSCpot=ts_smooth((scpot-(fitpars(0)*envR^2.+fitpars(1)*envR)),3)
EEnvSmooth=ts_smooth(fitpars(0)*envR^2.+fitpars(1)*envR+fitpars(2),3)
corrSCPOTN=scpot-reform(ElecCal2)
corrSCPOTN+=ts_smooth(reform(ElecCal2),1024)

;wavelet coherency analysis of the potential and the electric field
wavescpot=wavelet(scpot-supersine(phaseB,FittingParameters), timepotsecs(1)-timepotsecs(0),period=period)
waveEcorr=wavelet(ElecCal, timepotsecs(1)-timepotsecs(0),period=period,dj=dj,scale=scale)
wave_coherency,waveEcorr,timepotsecs,period,wavescpot,timepotsecs,period,wave_coher=wave_coher,wave_phase=wave_phase

;wavelet plots
!p.multi=[0,1,4]
window,xsize=700,ysize=1200,/free
contour, wave_coher,timepotsecs,1./period,/fill,/ylog,ytitle='Frequency (Hz)',xtitle='Time (s)'
xyouts,-0.1,5000,'(a)',/data

contour, wave_phase,timepotsecs,1./period,/fill,/ylog,ytitle='Frequency (Hz)',xtitle='Time (s)'
xyouts,-0.1,5000,'(b)',/data

meancoher=dblarr(96)
meanphase=dblarr(96)
for i=0,95 do begin
  meancoher(i)=mean(wave_coher(*,i))
  meanphase(i)=mean(wave_phase(*,i))
endfor


plot, 1./period, meancoher,/xlog,ystyle=8,ytitle='Coherence',xtitle='Frequency (Hz)',title='Mean Coherence/Phase difference Unshifted'
oplot, [1./(5*mean(deltaT_RC)),1./(5*mean(deltaT_RC))],[0,1],color=fsc_color('green'),line=3
xyouts,0.35,1.0,'(c)',/data
xyouts,20,0.2,'1/(5dt!DRC!N)',color=fsc_color('green'),/data
Axis, YAxis=1.0,yrange=[-90,90],/save,color=fsc_color('red'),ytitle='Phase Difference (deg)'
oplot, 1./period, meanphase,color=fsc_color('red')


waveEcorr1=waveEcorr
for i=0, 95 do begin
  for ii=0,8192-1 do begin
    waveEcorr1(ii,i)*=exp(dcomplex(0.0,1.0)*wave_coher(ii,i)*!PI/180.)
  endfor
endfor

;work out the cross spectrum (phase difference
C = CROSS_SPEC(ElecCal,scpot-supersine(phaseB,FittingParameters),deltat=timepotsecs(1)-timepotsecs(0),freq=freq,phase=phase,coherency=coherency,/double)

phases=[phase,-1.0*reverse(phase(1:4095))]

;do the phase shifting here
ElecPhaseShift=real_part(fft(fft(ElecCal)*exp(complex(0.0,-1.0)*phases),/inverse))
ElecPhaseShift2=real_part(fft(fft(ElecCal2)*exp(complex(0.0,-1.0)*phases),/inverse))

C = CROSS_SPEC(ElecPhaseShift,scpot,deltat=timepotsecs(1)-timepotsecs(0),freq=freq,phase=phase,coherency=coherency,/double)

;coherency of shifted signal
waveEcorrS=wavelet(ElecPhaseShift, timepotsecs(1),period=period,dj=dj,scale=scale)
wave_coherency,waveEcorrS,timepotsecs,period,wavescpot,timepotsecs,period,wave_coher=wave_coher,wave_phase=wave_phase

;mean coherence and phase values
meancoher=dblarr(96)
meanphase=dblarr(96)
for i=0,95 do begin
  meancoher(i)=mean(wave_coher(*,i))
  meanphase(i)=mean(wave_phase(*,i))
endfor


plot, 1./period, meancoher,/xlog,ystyle=8,xtitle='Frequency (Hz)',ytitle='Coherence',title='Mean Coherence/Phase difference shifted'
xyouts,0.35,1.0,'(d)',/data
oplot, [1./(5*mean(deltaT_RC)),1./(5*mean(deltaT_RC))],[0,1],color=fsc_color('green'),line=3
Axis, YAxis=1.0,yrange=[-100,100],/save,color=fsc_color('red'),ytitle='Phase Difference'
oplot, 1./period, meanphase,color=fsc_color('red')



;corrected for electric field, and phase shift
corrSCpotRawNew=(scpot-(ElecPhaseShift))
;corrected for e field and spin and phase shift
corrscpotnewspinrem=scpot-(ElecPhaseShift)-supersine(phaseB,FittingParameters)
;corrected for electric field and spin
corrSCPOTN=scpot-ElecCal-supersine(phaseB,FittingParameters)

meancoher1=dblarr(96)
meanphase1=dblarr(96)
for i=0,95 do begin
  meancoher1(i)=mean(wave_coher(*,i))
  meanphase1(i)=mean(wave_phase(*,i))
endfor

;calculations of different electron densities
;electric field removed and phase
term1corr=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
term2corr=(1.0d/((1.0d +(q*corrSCpotRawNew)/(kB*meanTE*11604.505d))))
term3corr=Iph0*exp(-q*double(corrSCpotRawNew)/(kB*TPh0*11604.505d))
term4corr=Iph1*exp(-q*double(corrSCpotRawNew)/(kB*TPh1*11604.505d))

;corrected for spin and phase and electric field
term1corrNEWCORR=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
term2corrNEWCORR=(1.0d/((1.0d +(q*corrscpotnewspinrem)/(kB*meanTE*11604.505d))))
term3corrNEWCORR=Iph0*exp(-q*double(corrscpotnewspinrem)/(kB*TPh0*11604.505d))
term4corrNEWCORR=Iph1*exp(-q*double(corrscpotnewspinrem)/(kB*TPh1*11604.505d))

nscorrNEWCORR=term1corrNEWCORR*term2corrNEWCORR*(term3corrNEWCORR+term4corrNEWCORR)




;
term1corrNEWSMOO=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
term2corrNEWSMOO=(1.0d/((1.0d +(q*ts_smooth(corrSCpotRawNew,smoothingwindow))/(kB*meanTE*11604.505d))))
term3corrNEWSMOO=Iph0*exp(-q*double(ts_smooth(corrSCpotRawNew,smoothingwindow))/(kB*TPh0*11604.505d))
term4corrNEWSMOO=Iph1*exp(-q*double(ts_smooth(corrSCpotRawNew,smoothingwindow))/(kB*TPh1*11604.505d))

nscorrNEWSMOO=term1corrNEWSMOO*term2corrNEWSMOO*(term3corrNEWSMOO+term4corrNEWSMOO)

;corrected for phase spin, electric field and final smoothing
term1corrNEWSMOOSpin=1.d/(q*ASpac)*sqrt((2.0D*!PI*me)/(kB*meanTE*11604.505d))
term2corrNEWSMOOSpin=(1.0d/((1.0d +(q*ts_smooth(corrscpotnewspinrem,smoothingwindow))/(kB*meanTE*11604.505d))))
term3corrNEWSMOOSpin=Iph0*exp(-q*double(ts_smooth(corrscpotnewspinrem,smoothingwindow))/(kB*TPh0*11604.505d))
term4corrNEWSMOOSpin=Iph1*exp(-q*double(ts_smooth(corrscpotnewspinrem,smoothingwindow))/(kB*TPh1*11604.505d))

nscorrNEWSMOOSpin=term1corrNEWSMOOSpin*term2corrNEWSMOOSpin*(term3corrNEWSMOOSpin+term4corrNEWSMOOSpin)

;results plot
!p.multi=[0,1,2]
window,xsize=800,ysize=1000,/free
plot, tscp-min(tscp),scpot,xtitle='Time (s)',ytitle='SC Potential (V)',yrange=[10,20]
oplot, tscp-min(tscp),ElecPhaseShift2,color=fsc_color('blue')

oplot, tscp-min(tscp),corrSCPOTN,color=fsc_color('cyan')
oplot, tscp-min(tscp),corrSCpotRawNew,color=fsc_color('orange')
oplot, tscp-min(tscp),ts_smooth(corrSCpotRawNew,smoothingwindow),color=fsc_color('red')

xyouts, 0.11,19.35,'V!Dsc!N',/data
xyouts, 0.1,18.7,string(fitpars(0),Format='(D0.6)')+'E!DCal shift!N!u2!N+'+string(fitpars(1),Format='(D0.3)')+'E!DCal shift!N+'+cgnumber_formatter(fitpars(2),decimal=3),color=fsc_color('blue'),/data
xyouts, 0.1,17.9,'No phase shifting',color=fsc_color('cyan'),/data
xyouts, 0.1,17.1,'V!Dsc sr!N-('+string(fitpars(0),Format='(D0.6)')+'E!DCal shift!N!u2!N+'+string(fitpars(1),Format='(D0.3)')+'E!DCal shift!N)',color=fsc_color('orange'),/data
xyouts, 0.1,16.3,'smooth(V!D corr!N)',color=fsc_color('red'),/data

xyouts,-0.2,20.0,'(a)',/data



plot, tscp-min(tscp),nsc*1e-6,xtitle='Time (s)',ytitle='n!De!N (cm!U-3!N)',yrange=[0.05,0.65]
;cgplot, tscp-min(tscp),nscorr*1e-6,color=fsc_color('red'),/overplot,/addcmd\
oplot,tne-min(tscp),n_e,color=fsc_color('forest green')
oplot, tne-min(tscp),n_epart2(*,10),color=fsc_color('green')
oplot, tscp-min(tscp),nscorrNEWCORR*1e-6,color=fsc_color('orange')
oplot, tscp-min(tscp),nscorrNEWSMOOSpin*1e-6,color=fsc_color('red')

xyouts,-0.2,.65,'(b)',/data
xyouts,0.1,0.3,'n!DSC!N',/data
xyouts,0.1,0.2,'n!DSC!N Corrected+smoothed',color=fsc_color('red'),/data
xyouts,0.1,0.25,'n!DSC!N Corrected',color=fsc_color('orange'),/data
xyouts,0.1,0.15,'n!De!N FPI-DES E>96eV',color=fsc_color('green'),/data
xyouts,0.5,0.15,'n!De!N FPI-DES moments',color=fsc_color('forest green'),/data


;relation between Electric field and spaceceft potential plot
!p.multi=0
window,xsize=800,ysize=500,/free
plot, findgen(150),polycurve1(findgen(150)+1,fitpars),xtitle='Electric Field Envelope (mV/m)',ytitle='Spacecraft Potential (V)',xrange=[0.0,150]
oplot, envR,scpot,color=fsc_color('blue'),psym=3
oplot,bincentres,binnedscpot,psym=7,color=fsc_color('red')
errplot,bincentres,binnedscpot+errorscpot,binnedscpot-errorscpot,color=fsc_color('red')
oplot, findgen(150),polycurve1(findgen(150)+1,fitpars),thick=4,color=fsc_color('red')
oplot, findgen(150),model,color=fsc_color('black'),thick=4



xyouts,5,5,string(fitpars(0),Format='(D0.6)')+'E!DEnv!N!u2!N+'+string(fitpars(1),Format='(D0.3)')+'E!DEnv!N+'+string(fitpars(2),Format='(D0.3)'),color=fsc_color('red'),/data
xyouts,5,2,string(result(1),Format='(D0.3)')+'E!DEnv!N+'+string(result(0),Format='(D0.3)'),color=fsc_color('black'),/data




end
