pro SpinRem

;run this program to ge the spin
;coefficients to go into supersine
;for removal of spin effect from sc potential
;the principle of the method can be read in 
;Torkar or Roberts 2017 JGR


;change local data directory and the deisred time range here
local_data_dir='C:\Users\oroberts\IDLWorkspace\'
trange=['2017-12-18 14:40:03.000','2017-12-18 14:50:03.000']
probes=['1']


;load edp data
mms_load_edp, trange=trange, probes=probes, level='l2', data_rate='fast', datatype='scpot', local_data_dir = local_data_dir,/time_clip
mms_load_edp, trange=trange, probes=probes, level='l2pre', data_rate='fast', local_data_dir = local_data_dir,/time_clip

;get the data
GET_DATA,'mms'+PROBES+'_edp_phase_fast_l2pre',TPH,PHASE
GET_DATA,'mms'+PROBES+'_edp_scpot_fast_l2',TSCP,SCPOT

;find local mean based on one spin
meanval=ts_smooth(scpot,640)
;remove local mean
scpotfluc=scpot-meanval


;plot phase vs local potential fluctuation
!p.multi=[0,1,2]
window
plot, phase, scpotfluc,psym=7,title='Potential Fluctuation',xrange=[0,360],ytitle='dV [V]',xtitle='Spacecraft Phase Angle [degrees]',xstyle=1

;define arrays for the angle bins
;and the arrays to hold the median
;values and the errors
anglebins=findgen(720)/2

medval=dblarr(730)
errori=dblarr(730)
binmid=findgen(730)/2.0+0.5

;bin the data and find median and error
for i=0,n_elements(anglebins)-2 do begin
  index=where(phase gt anglebins(i) and phase lt anglebins(i+1))
  ;use median value of the bin to limit the effects of outliers
  medval(i)=median(scpotfluc(index))
  ;calculate the error as 1/sqrt(2) *erfc^-1(0.5) * abs(x -median)
  errori(i)=1.4826*(median(abs(scpotfluc(index)-medval(i))))
   
endfor

;set a few points above 360 to the points near 0 such that the fitting does not
;produce a discontinuity as the phase angle goes from 360 to zero
medval(719)=medval(0)&errori(719)=errori(0)
medval(720)=medval(1)&errori(720)=errori(1)&medval(721)=medval(2)&errori(721)=errori(2)
medval(722)=medval(3)&errori(722)=errori(3)&medval(723)=medval(4)&errori(723)=errori(4)
medval(724)=medval(5)&errori(724)=errori(5)&medval(725)=medval(6)&errori(725)=errori(6)
medval(726)=medval(7)&errori(726)=errori(8)&medval(727)=medval(9)&errori(727)=errori(9)
medval(728)=medval(10)&errori(728)=errori(10)&medval(729)=medval(11)&errori(729)=errori(11)



;plot the binned data and the error bars
plot,binmid, medval,psym=7,xrange=[0,360],ytitle='dV [V]',xtitle='Spacecraft Spin Phase Angle [degrees]',title='Median of Potential Fluctuation',xstyle=1,yrange=[-0.6,0.6]

;initial guesses for fitting
p=fltarr(60)+1.0
 restore,'fittedspinparameters.sav'
 p=FittingParameters
FittingParameters=mpfitfun('supersine',binmid, medval,errori,p,maxiter=20000)

anglevec=lindgen(163840)*540./(8192.*20.)
;overplot the curve on the data
oplot, anglevec,supersine(anglevec,FittingParameters),color=FSC_Color('red')
save,FittingParameters,filename='fittedspinparameters.sav'
  end